"use strict";
/**
 * Timer
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Timer
 */
var Timer = /** @class */ (function () {
    /**
     * 생성자
     * @param interval interval 시간간격 안에 등록된 callback들이 차례대로 실행된다.
     * @param minTick callback과 callback사이에 최소 시간간격을 설정할 수 있다.
     */
    function Timer(interval, minTick) {
        this.interval = interval;
        this.minTick = minTick;
        this._workList = [];
        this._id = 0;
        this._t = null;
        if (!minTick) {
            this.minTick = 0;
        }
    }
    Timer.prototype.append = function (work) {
        var params = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            params[_i - 1] = arguments[_i];
        }
        var isStared = !!this._t;
        this.stop();
        this._workList.push({
            id: ++this._id,
            work: work,
            params: params,
        });
        if (isStared) {
            this.start();
        }
        return this._id;
    };
    Timer.prototype.remove = function (id) {
        var isStared = !!this._t;
        this.stop();
        var before = this._workList;
        this._workList = this._workList.filter(function (o) {
            return o.id !== id;
        });
        if (isStared) {
            this.start();
        }
        return before.length !== this._workList.length;
    };
    Timer.prototype.start = function () {
        this.stop();
        var count = (this._workList.length !== 0) ? this._workList.length : 1;
        var tick = this.interval / count;
        if (this.minTick) {
            tick = Math.max(tick, this.minTick);
        }
        this._t = setInterval(this._callback.bind(this), tick);
    };
    Timer.prototype.stop = function () {
        if (this._t) {
            clearInterval(this._t);
            this._t = null;
        }
    };
    Object.defineProperty(Timer.prototype, "isStarted", {
        get: function () {
            return !!this._t;
        },
        enumerable: true,
        configurable: true
    });
    Timer.prototype._nextWork = function () {
        var workPack = this._workList.shift();
        if (workPack) {
            this._workList.push(workPack);
        }
        return workPack;
    };
    Timer.prototype._callback = function () {
        var workPack = this._nextWork();
        if (workPack) {
            workPack.work.apply(workPack, workPack.params);
        }
    };
    return Timer;
}());
exports.Timer = Timer;
exports.default = Timer;

//# sourceMappingURL=index.js.map
