"use strict";
/**
 * test number
 */
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var fourdollar_1 = require("../fourdollar");
var timers_1 = require("timers");
describe('Timer', function () {
    it('Timer#start() && Timer#stop()', function (done) {
        var timer = new fourdollar_1.Timer(200, 100);
        var count = 0;
        timer.append(function () {
            count++;
            timer.stop();
        });
        timer.start();
        timers_1.setTimeout(function () {
            chai_1.expect(count).to.equal(1);
            chai_1.expect(timer.isStarted).to.be.false;
            timer.start();
        }, 300);
        timers_1.setTimeout(function () {
            chai_1.expect(count).to.equal(2);
            chai_1.expect(timer.isStarted).to.be.false;
            done();
        }, 600);
    });
    it('Timer#append() && Timer#remove()', function (done) {
        var timer = new fourdollar_1.Timer(200, 100);
        var count = 0;
        var id01 = timer.append(function (val) {
            count += val;
        }, 3);
        var id02 = timer.append(function (val) {
            count += val;
        }, 7);
        var id03 = timer.append(function (val) {
            count += val;
        }, 11);
        var id04 = timer.append(function (val) {
            count += val;
            timer.stop();
            chai_1.expect(count).to.equal(27);
            done();
        }, 13);
        timer.start();
        timer.remove(id02);
    });
    it('Timer > 0.2초에 한번씩 실행 된다.', function (done) {
        var timer = new fourdollar_1.Timer(200);
        var count = 0;
        timer.append(function (sDate) {
            var t = new Date().getTime() - sDate.getTime();
            chai_1.expect(Math.floor(t / 100)).to.equal(2 * ++count);
            if (count === 3) {
                timer.stop();
                done();
            }
        }, new Date());
        timer.start();
    });
    it('Timer > 0.5초에 모든 work들이 한번씩 실행 된다.', function (done) {
        var timer = new fourdollar_1.Timer(500);
        timer.append(function (sDate) {
            var t = (new Date().getTime() - sDate.getTime());
            chai_1.expect(Math.floor(t / 100)).to.equal(1);
        }, new Date());
        timer.append(function (sDate) {
            var t = (new Date().getTime() - sDate.getTime());
            chai_1.expect(Math.floor(t / 100)).to.equal(2);
        }, new Date());
        timer.append(function (sDate) {
            var t = (new Date().getTime() - sDate.getTime());
            chai_1.expect(Math.floor(t / 100)).to.equal(3);
        }, new Date());
        timer.append(function (sDate) {
            var t = (new Date().getTime() - sDate.getTime());
            chai_1.expect(Math.floor(t / 100)).to.equal(4);
        }, new Date());
        timer.append(function (sDate) {
            var t = (new Date().getTime() - sDate.getTime());
            chai_1.expect(Math.floor(t / 100)).to.equal(5);
            timer.stop();
            done();
        }, new Date());
        timer.start();
    });
    it('Timer > minTick', function (done) {
        var timer = new fourdollar_1.Timer(200, 100);
        timer.append(function (sDate) {
            console.log('first: ', sDate);
            var t = new Date().getTime() - sDate.getTime();
            chai_1.expect(Math.floor(t / 100)).to.equal(1);
        }, new Date());
        timer.append(function (sDate) {
            var t = new Date().getTime() - sDate.getTime();
            chai_1.expect(Math.floor(t / 100)).to.equal(2);
        }, new Date());
        timer.append(function (sDate) {
            console.log('third: ', sDate);
            var t = new Date().getTime() - sDate.getTime();
            chai_1.expect(Math.floor(t / 100)).to.equal(3);
        }, new Date());
        timer.append(function (sDate) {
            var t = new Date().getTime() - sDate.getTime();
            chai_1.expect(Math.floor(t / 100)).to.equal(4);
            timer.stop();
            done();
        }, new Date());
        timer.start();
    });
});

//# sourceMappingURL=timer.spec.js.map
