/**
 * test number
 */


import {expect} from 'chai'
import {
  Timer
} from '../fourdollar'
import { setTimeout } from 'timers';


describe('Timer', () => {
  it('Timer#start() && Timer#stop()', done => {
    const timer = new Timer(200, 100)
    let count = 0
    timer.append(() => {
      count++
      timer.stop()
    })
    timer.start()
    setTimeout(() => {
      expect(count).to.equal(1)
      expect(timer.isStarted).to.be.false
      timer.start()
    }, 300)
    setTimeout(() => {
      expect(count).to.equal(2)
      expect(timer.isStarted).to.be.false
      done()
    }, 600)
  })

  it('Timer#append() && Timer#remove()', done => {
    const timer = new Timer(200, 100)
    let count = 0
    let id01 = timer.append(val => {
      count += val
    }, 3)
    let id02 = timer.append(val => {
      count += val
    }, 7)
    let id03 = timer.append(val => {
      count += val
    }, 11)
    let id04 = timer.append(val => {
      count += val
      timer.stop()
      expect(count).to.equal(27)
      done()
    }, 13)
    timer.start()
    timer.remove(id02)
  })

  it('Timer > 0.2초에 한번씩 실행 된다.', done => {
    const timer = new Timer(200)
    let count = 0
    timer.append((sDate: Date) => {
      let t = new Date().getTime() - sDate.getTime()
      expect(Math.floor(t / 100)).to.equal(2 * ++count)
      if(count === 3) {
        timer.stop()
        done()
      }
    }, new Date())
    timer.start()
  })

  it('Timer > 0.5초에 모든 work들이 한번씩 실행 된다.', done => {
    const timer = new Timer(500)
    timer.append((sDate: Date) => {
      let t = (new Date().getTime() - sDate.getTime())
      expect(Math.floor(t / 100)).to.equal(1)
    }, new Date())
    timer.append((sDate: Date) => {
      let t = (new Date().getTime() - sDate.getTime())
      expect(Math.floor(t / 100)).to.equal(2)
    }, new Date())
    timer.append((sDate: Date) => {
      let t = (new Date().getTime() - sDate.getTime())
      expect(Math.floor(t / 100)).to.equal(3)
    }, new Date())
    timer.append((sDate: Date) => {
      let t = (new Date().getTime() - sDate.getTime())
      expect(Math.floor(t / 100)).to.equal(4)
    }, new Date())
    timer.append((sDate: Date) => {
      let t = (new Date().getTime() - sDate.getTime())
      expect(Math.floor(t / 100)).to.equal(5)
      timer.stop()
      done()
    }, new Date())
    timer.start()
  })

  it('Timer > minTick', done => {
    const timer = new Timer(200, 100)
    timer.append((sDate: Date) => {
      console.log('first: ', sDate)
      let t = new Date().getTime() - sDate.getTime()
      expect(Math.floor(t / 100)).to.equal(1)
    }, new Date())
    timer.append((sDate: Date) => {
      let t = new Date().getTime() - sDate.getTime()
      expect(Math.floor(t / 100)).to.equal(2)
    }, new Date())
    timer.append((sDate: Date) => {
      console.log('third: ', sDate)
      let t = new Date().getTime() - sDate.getTime()
      expect(Math.floor(t / 100)).to.equal(3)
    }, new Date())
    timer.append((sDate: Date) => {
      let t = new Date().getTime() - sDate.getTime()
      expect(Math.floor(t / 100)).to.equal(4)
      timer.stop()
      done()
    }, new Date())
    timer.start()
  })
})