/**
 * Timer
 */


type WorkPack = {
  id: number
  work: Function
  params: any[]
}


/**
 * Timer
 */
export class Timer {
  private _workList: WorkPack[] = []
  private _id: number = 0
  private _t: any = null

  /**
   * 생성자
   * @param interval interval 시간간격 안에 등록된 callback들이 차례대로 실행된다.
   * @param minTick callback과 callback사이에 최소 시간간격을 설정할 수 있다.
   */
  constructor(public readonly interval: number, public readonly minTick?: number) {
    if(!minTick) {
      this.minTick = 0
    }
  }

  append(work: Function, ...params: any[]): number {
    let isStared = !!this._t
    this.stop()
    this._workList.push({
      id: ++this._id,
      work,
      params,
    })
    if(isStared) {
      this.start()
    }
    return this._id
  }

  remove(id: number): boolean {
    let isStared = !!this._t
    this.stop()
    let before = this._workList
    this._workList = this._workList.filter(o => {
      return o.id !== id
    })
    if(isStared) {
      this.start()
    }
    return before.length !== this._workList.length
  }

  start(): void {
    this.stop()
    let count = (this._workList.length !== 0)? this._workList.length : 1
    let tick = this.interval / count
    if(this.minTick) {
      tick = Math.max(tick, this.minTick)
    }
    this._t = setInterval(this._callback.bind(this), tick)
  }

  stop(): void {
    if(this._t) {
      clearInterval(this._t)
      this._t = null
    }
  }

  get isStarted(): boolean {
    return !!this._t
  }

  private _nextWork(): WorkPack {
    let workPack = this._workList.shift()
    if(workPack) {
      this._workList.push(workPack)
    }
    return workPack
  }

  private _callback(): void {
    let workPack = this._nextWork()
    if(workPack) {
      workPack.work(...workPack.params)
    }
  }
}

export default Timer
